function [M,L,S] = LS_decomposition(d,matA,epsilon)

% CZ
% Tato funkce rekonstruuje namen video. Rekonstrukce pedpokld, e video M se d modelovat jako souet dvou matic L a S, 
% kde L je matice s nzkou hodnost a S je dk matice ve spektru po dcch, tj. M = L + S. Podrobnj informace viz lnek O. Ricardo, C. Emmanuel, 
% S. Daniel: Low-Rank and Sparse Matrix Decomposition for Accelerated Dynamic MRI with Separation of Background and Dynamic 
% Components.
% Vstupn parametry funkce jsou:
%     d ... namen matice, ve kter jsou ve sloupcch naskldan fourierovy obrazy jednotlivch snmk videa
%     matA ... mic matice, pomoc kter byla namena matice d
%     epsilon ... pesnost vpotu - algoritmus kon, kdy je relativn zmna rekonstrukce men ne epsilon
% Vstupn parametry jsou:
%     M ... rekonstruovan matice videa tentokrt ji v prostorov oblasti, ve sloupcch m naskldan jednotliv snmky
%     L ... vsledn matice s nzkou hodnost (uspodan stejn jako M)
%     S ... vsledn dk matice ve spektru po dcch (uspodan stejn jako M)
% Pouit: [M,L,S] =  LS_decomposition(d,A,epsilon)  
% ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
% EN
%This function reconstructs measured video. Reconstruction assumes that video M can be modeled as a sum of two matrix L and S, 
% where L low rank matrix and S is sparse matrix in row frequency domain, i.e. M = L + S. More details in article O. Ricardo, 
% C. Emmanuel, S. Daniel: Low-Rank and Sparse Matrix Decomposition for Accelerated Dynamic MRI with Separation of Background 
% and Dynamic Components.
% Input parameters are:
%     d ... measured matrix; in each column of d is fourier transform of one video frame   
%     matA ... measure matrix, using this matrix was measured d
%     epsilon ... accurancy of computation - algorithm ends when relative change of reconstruction is smaller than epsilon
% Output parameters are:
%     M ... reconstructed matrix in spatial area, in each column of d is one video frame 
%     L ... final low rank matrix (same organization as M)
%     S ... final sparse matrix in row frequency domain (same organization as M)
% Use: [M,L,S] =  LS_decomposition(d,A,epsilon)  

    n = sqrt(size(d,1));
    t = size(d,2);

    % funkce mkkho prahovn / function of soft tresholding
    soft = @(z, T) sign(z).*max(abs(z)-T, 0);
    
    % mic funkce / measure function
    A = @(x) matA.*x;

    % nastaven potench hodnot / initialization
    M = A(d);

    for i = 1:t
        M(:,i) = reshape(ifft2(reshape(M(:,i),n,n)),[],1); 
    end

    S = zeros(size(M));
    pom = zeros(size(M));

    iterace = true;
    poc = 0;

    % vlastn rekonstrukce / main computation
    while iterace
    
        Mstare = M;
    
        % aktualizace L (matice s nzkou hodnost) / updating L (low rank matrix)
        [U,Sigma,V] =  svd(M-S,'econ');
        sigma = diag(Sigma);
        sigma = soft(sigma,0.01*max(sigma));
        r = sum(sigma > 0);
        U = U(:,1:r); V = V(:,1:r); Sigma = diag(sigma(1:r));
        L = U*Sigma*V';
    
        % aktualizace S (dk matice) / updating S (sparse matrix)
        S = fft((M-L)');
        S = (ifft(soft(S,0.01*max(max(abs(S))))))';
    
        % aktualizace M / updating M
        M = L+S;
    
        for i = 1:t
            pom(:,i) = reshape(fft2(reshape(M(:,i),n,n)),[],1); 
        end
    
        pom = A(A(pom) - d);
    
        for i = 1:t
            pom(:,i) = reshape(ifft2(reshape(pom(:,i),n,n)),[],1); 
        end
    
        M = M - pom;
    
        % kontrola konvergence / check of convergence
        if norm(Mstare - M)/norm(Mstare) < epsilon
            iterace = false;
        end
    
        poc = poc+1;
        display([num2str(poc),'. iterace / iteration, relativn zmna / relative change ', num2str(norm(Mstare - M)/norm(Mstare))])
    end
    
end
