function maska = mask(n, typ, param)

% CZ
% Tato funkce vytv mic matici (masku) na men jednoho snmku videa.
% Vstupn parametry jsou:
%     n ... rozmr snmku
%     typ ... druh masky: 'nrand' (nerovnomrn nhodn - vy pst v potku), 'radial' (radiln ry), 'random' (nhodn) nebo 
%               'spiral' (spirly)
%     (param) ... nepovinn parametr, zle na druhu masky: 
%                     'nrand' - param.pst ... piblin procento namench vzork, defaultn 0.5 
%                               - param.ctverec ...  tvercov vrstevnice pravdpodobnosti, defaultn false
%                     'radial' - param.pocet ... poet pmek, defaultn round(6*n/10)
%                     'random' - param.probability ... pravdpodobnost namen koeficientu, 0 < param < 1 (pi param.conj == true m smysl 
%                                                                  pouze do 0.5), defaultn 0.5 
%                                  - param.conj ... zamezen vbru komplexn sdruench koeficient (pi hodnot true), defautn false     
%                     'spiral' - param.number ... poet spirl, defaultn round(4*n/10)
%                               - param.p ... parametr "tvercovatn" spirly pomoc p normy, 1 <= param.p < infty, defaultn 2
%                               - param.a ... vzdlenost potku spirly od jejho plu, defaultn nhodn rand + 0.5
%                               - param.b ... nrst polomru (hel tenho vektoru) spirly, defaultn nhodn rand + 0.1
%                     param.procento ... zobraz procento namench vzork (u vech druh masek), dafaultn false 
%  Pouit: maska = mask(n, typ)
%              maska = mask(n, typ, param)              
% ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
% EN
% This function creates measuring matrix (mask) to measure one video frame.
% Input parameters are:
%     n ... size of square frames
%     typ ... kind of mask: 'nrand' (random but not uniformly distributed - higher density in origin), 'radial' (radial lines), 'random' or 
%               'spiral' (spirals)
%     (param) ... optional parameter depends on kind of mask: 
%                     'nrand' - param.pst ... rough percent of measured data, defaut 0.5
%                               -  param.ctverec ... square contour line of probability, default false
%                     'radial' - param.pocet ... number of lines, default round(6*n/10)
%                     'random' - param.probability ...  probability of data measurement, 0 < param < 1 (in case of param. conj == true it has 
%                                                                   sence only to value 0.5), default 0.5 
%                                  - param.conj ... avoid choosing conjugate coefficients (in case true), defaut false     
%                     'spiral' - param.number ... number of spirals, default round(4*n/10)
%                               - param.p ... parameter of "squaring" spiral using p norm, 1 <= param.p < infty, default 2
%                               - param.a ... distance between beginning of spiral and its pole, default randomly rand + 0.5
%                               - param.b ... increase the radius (angle of the tangent vector), default randomly rand + 0.1
%                     param.procento ... shows percent of measured data (for all types of mask), dafault false 
% Use: maska = mask(n, typ)
%         maska = mask(n, typ, param)      

%  2012 - 2013 Marie Dakov, Radek Hrbek

    switch typ
        
        % nhodn maska s nerovnomrnm rozdlenm (vt pravdpodobnost smrem k potku k-prostoru) 
        %/ random mask with not uniform distribution (bigger density in the coordinate origin of k-space)
        case 'nrand'
            % nastaven nepovinnch parametr / setting up optional parameters
            if nargin < 3, param = struct; end 
            
            % piblin procento namench vzork / rough percent of measured data
            if ~isfield(param, 'pst'), param.pst = 0.5; end 
            
            % tvercov vrstevnice pravdpodobnosti / square contour line of probability
            if ~isfield(param, 'ctverec'), param.ctverec = false; end 
            
            % s, ve kter se vysluje hustota psti / grid in which is calculated probability density
            sit = linspace(-3,3,n); 

            % hustota pravdpodobnosti standardnho normlnho rozdlen / probability distribution function of standard normal distribution
            nrozd = @(x) exp(-x.^2/2)/(sqrt(2*pi)); 

            % vpoet pravdpodobnost pro namen jednotlivch bod / calculate the probability for measuring individual points
            h1 = nrozd(sit); 

            % korekce tto pravdpodobnosti / correction of this probability
            if param.ctverec
                h1 = min((h1 + 0.6995),1); % zven psti / increase of probability
                h1 = h1.^3; % zven rychlosti klesn smrem ke krajm / modification of probab. slope 
                h2 = ones(n,1)*h1; % pichastn k vpotu tvercovch vrstevnic psti
                h3 = h1'*ones(1,n);

                hust = min(h2,h3); % vsledn pst / resultant probability
            else   
                h2 = h1'*h1; % dvojrozmrn hustota psti normlnho rozdlen / 2D pdf of normal distribution

                h3 = min((h2 + 0.859756),1); % zven psti / increase of probability
                hust = h3.^6; % zven rychlosti klesn smrem ke krajm / modification of probab. slope 
            end
 
            maska = rand(n); % generovn masky
            maska = (maska > 1 - hust - (param.pst-0.5) );

        % radiln maska / radial mask
        case 'radial'
            % nastaven nepovinnch parametr / setting up optional parameters
            if nargin < 3, param = struct; end 
            
            if ~isfield(param, 'pocet'), param.pocet = round(6*n/10); end % poet pmek / number of lines   
            
            maska = zeros(n);
             
            phi = sort(-pi/2 + pi.*rand(param.pocet,1)); % hel / angle
            for i = 1:length(phi)
                for x = 1:0.01:n
                    y = n/2 + round((x - n/2) * tan(phi(i))); % vpoet bodu na pmce / computation of point on line
                    if y <= n && y > 0
                        maska(round(y), round(x)) = 1;
                    end
                 end
            end

        % nhodn maska / random mask
        case 'random'
            % nastaven nepovinnch parametr / setting up optional parameters
            if nargin < 3, param = struct; end

            % pravdpodobnost namen bodu / probability of data measurement
            if ~isfield(param, 'probability'), param.probability = 0.5; end 
            
            % zamezen vbru komplexn sdruench koeficient / avoid choosing conjugate coefficients
            if ~isfield(param, 'conj'), param.conj = false; end 
            
            if param.conj
                maska = zeros(n);
                if mod(n,2) == 0
                    maska(1:n/2+1,1:n/2+1) = rand(n/2+1);
                    maska(2:n/2,n/2+2:n) = rand(n/2-1);
                    maska = (maska > 1 - 2*param.probability); % dvakrt kvli restrikci / twice as many due to restriction
                else
                    maska(1:(n+1)/2,1:(n+1)/2) = rand((n+1)/2);
                    maska(2:(n+1)/2,(n+1)/2+1:n) = rand((n-1)/2);
                    maska = (maska > 1 - 2*param.probability); % dvakrt kvli restrikci / twice as many due to restriction
                end
            else
                maska = rand(n);
                maska = (maska > 1 - param.probability);
            end

        % spirln maska / spiral mask
        case 'spiral'
            % nastaven nepovinnch parametr / setting up optional parameters
            if nargin < 3, param = struct; end
            
            if ~isfield(param, 'number'), param.number = round(4*n/10); end % poet spirl / number of spirals
            if ~isfield(param, 'p'), param.p = 2; end % p norma / p norm
            if ~isfield(param, 'a'), param.a = rand + 0.5; end % vzdlenost potku od plu / distance between beginning and pole
            % nrst polomru (hel tenho vektoru) / increase the radius (angle of the tangent vector)
            if ~isfield(param, 'b'), param.b = rand + 0.1; end 
              
            maska = zeros(n);
           
            maska(round(n/ 2), round(n/2)) = 1;
            
            for i = 0:2*pi/param.number:(param.number-1)*2*pi/param.number    
                for phi = 0:pi/500:200*pi;
                    r = param.a*exp(param.b * phi); % vpoet bodu spirly v polrnch souadnicch / computation of spiral point in polar coordinates
                    [x, y] = pol2cart(phi + i, r); % pevod do kartzsk soustavy / transform to cartesian coordinates
                    x = norm([x y])*x/norm([x y],param.p); % korekce pomoc p normy, aby byla spirla i v rozch / corection using p norm
                    y = norm([x y])*y/norm([x y],param.p);
                    x = round(n/2 + x);
                    y = round(n / 2 + y);
                    if (x <= 0 || x > n || y <= 0 || y > n)
                        break;
                    else
                        maska(y, x) = 1;
                    end
                end 
            end
    end

    if ~isfield(param, 'procento'), param.procento = false; end
    if param.procento
        pocet1 = sum(sum(maska))/(n^2)*100;
        display(['poet namench koeficient je / number of measured coefficient is ', num2str(pocet1), ' %'])
    end
    
end

